#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libappindicator/app-indicator.h>
#include "states.h"
#include "statusicon.h"

GtkWidget *mainwindow;
AppIndicator *indicator;
States status_icon_currentstate;

static void activate_action (GtkAction *action);

static GtkActionEntry entries[] = {
  {"FileMenu",NULL,"_File"},
  {"New","document-new","_New","<control>N","Create a new file",G_CALLBACK(activate_action)},
  {"Open","document-open","_Open","<control>O","Open a file",G_CALLBACK(activate_action)},
  {"Save","document-save","_Save","<control>S","Save file",G_CALLBACK(activate_action)},
  {"Close","window-close","_Close","<control>W","Close the window",G_CALLBACK(activate_action)},
  {"Quit","application-exit","_Quit","<control>Q","Exit the application",G_CALLBACK(activate_action)},
};
static guint n_entries = G_N_ELEMENTS (entries);

static const gchar *ui_info =
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='New'/>"
"      <menuitem action='Open'/>"
"      <menuitem action='Save'/>"
"      <separator/>"
"      <menuitem action='Close'/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"  </menubar>"
"  <popup name='IndicatorPopup'>"
"    <menuitem action='New' />"
"    <menuitem action='Open' />"
"    <menuitem action='Save' />"
"    <menuitem action='Close' />"
"    <menuitem action='Quit' />"
"  </popup>"
"</ui>";

static void activate_action(GtkAction *action) {
  const gchar *name = gtk_action_get_name(action);
  if(strcmp(name,"Quit") == 0 || strcmp(name,"Close") == 0) {
    if(mainwindow != NULL) {
      gtk_window_close(GTK_WINDOW(mainwindow));
      return;
    }
  }
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new(NULL,GTK_DIALOG_DESTROY_WITH_PARENT,GTK_MESSAGE_INFO,GTK_BUTTONS_CLOSE,"You activated action: \"%s\"",name);
  g_signal_connect(dialog,"response",G_CALLBACK(gtk_widget_destroy),NULL);
  gtk_widget_show(dialog);
}

void status_icon_create(GtkWidget *window, char *statusoff, char *statuson) {
  if(indicator == NULL) {
    mainwindow = window;
    GtkUIManager *uim;
    GError *error = NULL;
    GtkWidget *indicator_menu;
    GtkActionGroup *action_group;

    action_group = gtk_action_group_new("AppActions");
    gtk_action_group_add_actions(action_group,entries,n_entries,window);

    uim = gtk_ui_manager_new();
    g_object_set_data_full(G_OBJECT(window),"ui-manager",uim,g_object_unref);
    gtk_ui_manager_insert_action_group(uim,action_group,0);
    gtk_window_add_accel_group(GTK_WINDOW(window),gtk_ui_manager_get_accel_group(uim));

    if (!gtk_ui_manager_add_ui_from_string(uim,ui_info,-1,&error)) {
      g_message("Failed to build menus: %s\n",error->message);
      g_error_free(error);
      error = NULL;
    }
    
    indicator = app_indicator_new("melonvpn",statusoff,APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
    indicator_menu = gtk_ui_manager_get_widget(uim,"/ui/IndicatorPopup");
    app_indicator_set_status(indicator,APP_INDICATOR_STATUS_PASSIVE);
    app_indicator_set_attention_icon(indicator,statuson);
    app_indicator_set_menu(indicator,GTK_MENU(indicator_menu));
  }
}

void status_icon_set_state(AppIndicatorStatus status) {
  app_indicator_set_status(APP_INDICATOR(indicator),status);
}
